@@ -1,59 +1,42 @@
 #line 2 "edit-distance.c.tmpl"
 #include <stdio.h>
-/* For INT_MAX/INT_MIN */
 #include <limits.h>
-/* For malloc. */
 #include <stdlib.h>
+#include <stdint.h>
 
 #include "config.h"
 #include "text-fuzzy.h"
 #include "edit-distance-int.h"
-#line 12 "edit-distance-int.c"
-int distance_int (
-                    text_fuzzy_t * tf)
 
+int distance_int (text_fuzzy_t * tf)
 {
-
-
-
-
-#line 127 "edit-distance.c.tmpl"
-#line 129 "edit-distance.c.tmpl"
-
-    const unsigned int * word1 = (const unsigned int *) tf->b.unicode;
+    const int * word1 = (const int *) tf->b.unicode;
     int len1 = tf->b.ulength;
-    const unsigned int * word2 = (const unsigned int *) tf->text.unicode;
+    const int * word2 = (const int *) tf->text.unicode;
     int len2 = tf->text.ulength;
     /* Return value. */
     int d;
     /* X and Y coordinates in the matrix of strings. */
     int i;
     int j;
-#line 142 "edit-distance.c.tmpl"
+    /* Unfeasible value; indicates no match. */
+    int large_value;
     int max;
-#line 145 "edit-distance.c.tmpl"
-#line 271 "edit-distance.c.tmpl"
 
     /* Matrix is the dynamic programming matrix. We economize on space
        by having only two columns. */
 
-#ifdef __GNUC__
+#ifdef STACKALLOCOK
     int matrix[2][len2 + 1];
 #else
     int * matrix[2];
 #endif
-    int large_value;
 
     max = tf->max_distance;
 
-#ifndef __GNUC__
+#ifndef STACKALLOCOK
     for (i = 0; i < 2; i++) {
-	matrix[i] = calloc (len2 + 1, sizeof (int));
-	if (! matrix[i]) {
-	    fprintf (stderr, "%s:%d: calloc failed.\n",
-		     __FILE__, __LINE__);
-	    return -1;
-	}
+	CALLOC (matrix[i], len2 + 1, sizeof (int));
     }
 #endif
 
@@ -85,7 +68,7 @@ int distance_int (
 
     /* Loop over column. */
     for (i = 1; i <= len1; i++) {
-        unsigned int c1;
+        int c1;
         /* The first value to consider of the ith column. */
         int min_j;
         /* The last value to consider of the ith column. */
@@ -100,6 +83,9 @@ int distance_int (
         c1 = word1[i-1];
         min_j = 1;
         max_j = len2;
+	/* If we have a maximum permitted distance, we can set the
+	   minimum and maximum columns to inspect to be smaller than
+	   the smallest and largest values respectively. */
         if (max != NO_MAX_DISTANCE) {
             if (i > max) {
                 min_j = i - max;
@@ -124,14 +110,13 @@ int distance_int (
                 matrix[next][j] = large_value;
             }
             else {
-                unsigned int c2;
+                int c2;
 
-                c2 = word2[j-1];
+                c2 = word2[j - 1];
                 if (c1 == c2) {
                     /* The character at position i in word1 is the same as
                        the character at position j in word2. */
-                    matrix[next][j] = matrix[prev][j-1];
-
+                    matrix[next][j] = matrix[prev][j - 1];
                 }
                 else {
                     /* The character at position i in word1 is not the
@@ -167,27 +152,19 @@ int distance_int (
                    maximum, so no match less than or equal to max can be
                    found by looking at succeeding columns. */
 
-#ifndef __GNUC__
-		for (i = 0; i < 2; i++) {
-		    free (matrix[i]);
-		}
-#endif
-                return large_value;
+		d = large_value;
+		goto cleanup;
             }
         }
     }
     d = matrix[len1 % 2][len2];
 
-#ifndef __GNUC__
+    cleanup:
+
+#ifndef STACKALLOCOK
     for (i = 0; i < 2; i++) {
 	free (matrix[i]);
     }
 #endif
-#line 425 "edit-distance.c.tmpl"
     return d;
 }
-
-/* Some people are not using GCC-compatible compilers. See, for
-   example, this test failure.
-   http://ppm4.activestate.com/sun4-solaris-64/5.12/1200/B/BK/BKB/Text-Fuzzy-0.11.d/log-20130328T025706.txt */
-
