@@ -543,8 +543,6 @@ idic_init_dic (idic_t * dic, const int * word1, int len1)
     /* There should not be any zeros in word1. */
     prev = 0;
     for (i = 0; i < len1; i++) {
-	int j;
-	int found;
 	int key;
 
 	key = worduniq[i];
@@ -643,12 +641,6 @@ adic_reset_dic (adic_t * dic)
     return 0;
 }
 
-int
-adic_free_dic (adic_t * dic)
-{
-    return 0;
-}
-
 int
 adic_find (adic_t * dic, uint8_t key)
 {
